/*
 * Copyright (c)1997 by Dianne Hackborn.
 * All rights reserved.
 *
 * Based (increasingly loosly) on WebTerm, a Web-based terminal
 * applet written in Java, which is
 * Copyright (C)1996 by the National Alliance for Computational
 * Science and Engineering (NACSE).
 * See <URL:http://www.nacse.org/> for more information.
 *
 * This code is not public domain, nor freely distributable.
 * Please direct any questions or requests to Dianne Hackborn,
 * at <hackbod@lucent.com> or <hackbod@enteract.com>.
 *
 * ----------------------------------------------------------------------
 *
 * ArpTerminalMsg.h
 *
 * This file defines the message protocol between the terminal
 * and a remote device.  It is based on the BMessage: the two
 * objects talk with each other through BMessenger handles.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 *
 * 0.1: Created this file.
 *
 */

#pragma once

#ifndef ARPCOMMON_TERMINALMSG_H
#define ARPCOMMON_TERMINALMSG_H

// The messages we send or receive

enum {
	/**	Text going from or coming to the terminal.  Data fields are:
	 **
	 **	"text" (B_ASCII_TYPE): The ASCII text being transmitted;
	 **	there may be multiple text data items in an array.
	 **	Note that it is perfectly acceptable for this text to
	 **	contain null ('\0') characters, so you CAN NOT use standard
	 **	C string manipulation functions on it!
	 **/
	TERM_XFER_TEXT_MSG = 'tXFR',
	
	/**	Inform remote device that terminal has attached to it.
	 **
	 ** This message is sent to the remote device, when it is
	 ** attached to the terminal by calling SetRemote().  It is
	 ** primarily intended to supply the remote with the
	 ** terminal's BMessenger, so that full communication can
	 ** proceed.  Data fields are:
	 **
	 ** "terminal" (B_MESSENGER_TYPE): The messenger used for
	 ** sending messages to the terminal.
	 **/
	TERM_ATTACH_MSG = 'tATC',
	
	/**	Request terminal status.
	 **
	 **	This command is sent to the terminal; it asks for the terminal
	 **	to send messages back containing its state information.
	 **	Currently, this means sending back the following messages:
	 **	
	 **		TERM_WINDOW_SIZE_MSG
	 **		TERM_EMULATORS_MSG
	 **	
	 ** There are no defined data fields for this message.
	 **/
	TERM_REQUEST_STATUS_MSG = 'tSTS',

	/**	Transmit terminal size.
	 **
	 **	This command is sent by the terminal to the data sink,
	 **	describing the current terminal dimensions.  It is trasmited
	 **	whenever these dimensions change, or in response to a
	 **	TERM_REQUEST_STATUS_MSG.
	 **
	 ** You can also send this message to the terminal to change
	 ** its dimensions; these dimensions are the same as the
	 ** TermSetFixedSize command, that is -1 one in a coordinate
	 ** specify that coordinate should follow the window size.
	 **
	 **	Data fields are:
	 **	
	 **	"columns" (B_INT32_TYPE) the number of columns in characters.
	 **	"rows" (B_INT32_TYPE) the number of rows in characters.
	 **/
	TERM_WINDOW_SIZE_MSG = 'tWSZ',
	
	/**	Transmit terminal emulators.
	 **
	 **	This command is sent by the terminal to the data sink,
	 **	describing the current terminal emulator types available.
	 ** It is transmited when the terminal attaches to the device
	 ** or in response to a TERM_REQUEST_STATUS_MSG.
	 **
	 ** Data fields are:
	 **
	 ** "name" (B_STRING_TYPE) an array of the emulator names
	 ** available; these are the same names described in RFCxxxx.
	 **
	 ** "longname" (B_STRING_TYPE) array of more descriptive names,
	 ** corresponding to the names in the above array.  These are
	 ** the names that should be used when displaying text to the
	 ** user.
	 **/
	TERM_EMULATORS_MSG = 'tEMU',
	
	/**	Select terminal emulation.
	 **
	 **	This command is sent to the terminal to switch to
	 ** a particular terminal emulation.  As soon as the terminal
	 ** receives this message, it creates a new emulator of the
	 ** given type (if one can be found) and starts using it,
	 ** deallocating any emulator that it had previously been
	 ** using.
	 **
	 ** This me message will also be sent by the terminal to
	 ** the remote when its emulation changes.
	 **
	 ** Data fields are:
	 **
	 ** "name" (B_STRING_TYPE) an array of the emulator names
	 ** available desired.  The terminal starts at index zero,
	 ** searching through the names until the first one it finds
	 ** for which it has an emulation.  If none match, the
	 ** terminal will fall back on its default "dumb" terminal.
	 **/
	TERM_SELECT_EMULATION_MSG = 'tSMU',
};

#endif
